import javax.swing.*;
import javax.swing.event.*; // ChangeListener
import java.awt.*; // Container
import java.net.*;
import java.io.*; // OutputStream and InputStream
import java.util.*; // Iterator
import javax.imageio.*;
import java.awt.image.*;


public class Control extends JApplet implements ChangeListener, Runnable{
Container cp; // ContentPane
JSlider js;
Window window;

String server="";	// server name
int port=0;	// port number
SocketAddress addr=null;	// address where we will connect to

Socket sock=null;
OutputStream sout=null;	// socket output stream ( write here )
InputStream sin=null;
ObjectInputStream objIn;

int sliderVal=0;		// if new value is not much different than oldVal then don't send it...
ImageReader ir;
byte imgBuff[];


	public void init() {
		setBackground(Color.BLACK);
		cp = getContentPane();
		cp.setLayout(new BorderLayout());
		js = new JSlider(0,255);
		js.setEnabled(false);
		cp.add(js,BorderLayout.SOUTH);
		window = new Window();	// we need to create a panel to paint the image onto
		cp.add(window,BorderLayout.CENTER);	// this is where the image will go

		try{
			server = getCodeBase().getHost(); // or use getParameter("Server");
			port = Integer.parseInt(getParameter("Port"));
		} catch(Exception e){
			System.out.println("Could not parse port number: "+e);
			return;
		}
		System.out.println("Server: " + server + " Port: " +port);
		addr = new InetSocketAddress(server, port);
	}


	public void start(){
		System.out.println("Starting...");
		sock = new Socket();
		try{
			sock.setSoTimeout(10*1000); // 10 seconds
			sock.connect(addr); // if we use connect, we can control timeout
			sock.setSoTimeout(0); // read and write should not timeout
			sout = sock.getOutputStream();
			sin = sock.getInputStream();
			objIn = new ObjectInputStream(sin);
		}catch(Exception e){
			System.out.println("Error Connecting to " +server+":"+port+" "+e);
			stop();
			return;
		}
		System.out.println("Connected to " +server+ ":" +port);
		js.addChangeListener(this);
		js.setEnabled(true);

		System.out.println("Creating Jpeg Reader...");
		Iterator it = ImageIO.getImageReadersByFormatName("jpeg");
		if(!it.hasNext()){
			System.out.println("Could not get an image reader for jpeg - exiting...");
			stop();
			return;
		}
		ir = (ImageReader) it.next();
		ImageIO.setUseCache(false); // do not create a temp file
		new Thread(this).start();
	}

	public void run(){
		System.out.println("Starting thread...");
		while(true){
			try{
				int bsize = objIn.readInt(); // read data size
				System.out.println("File size = "+bsize);
				if(imgBuff==null || imgBuff.length < bsize){ 
					System.out.println("Resizing buffer to "+bsize);
					imgBuff = new byte[bsize];
				}
				objIn.readFully(imgBuff,0,bsize); // read the actuall data
				ByteArrayInputStream arrin = new ByteArrayInputStream(imgBuff);
				ir.setInput(ImageIO.createImageInputStream(arrin),false); // not forward only
				BufferedImage img = ir.read(0); // read first image in the stream
				if(img!=null){
					System.out.println("Setting image...");
					window.setImage(img);
				} else {
					System.out.println("Image is null...");
				}
				sout.write(sliderVal); // send the slider value to the server
			}catch(IIOException imge){
				System.out.println("Error decoding image: ["+imge+ "] continuing...");
			}
			catch(Exception e){
				System.out.println("Error while reading/writing socket or decoding image: "+e);
				stop();	// once we loose connection to server it's like stoppint the applet
				break; // connection reset by peer?
			}
		}
		System.out.println("Exiting thread...");
	}


	public void stop(){
		System.out.println("Stopping...");
		window.setImage(null);
		try{
			objIn.close();
			sin.close();
			sout.close();
			sock.close();
		} catch(Exception e){
			System.out.println("Cannot close socket: "+e);
		}
		sock=null;
		js.setEnabled(false);
		js.removeChangeListener(this); // it will stop the events dispatch
		showStatus("Disconnected...");
	}


	public void  stateChanged(ChangeEvent e){ // implement the ChangeListener
		sliderVal = (byte) ((JSlider) e.getSource()).getValue();
		System.out.println("Slider value: " +sliderVal);
	}
}