import java.io.*;
import java.net.*;
//import java.lang.*;

class Jft639 implements Runnable{
static int EOF = -1; // it's not defined but returned by read()

static int CMD_SHORT_PULSE=85; // servo will make 90 degree turns
static int CMD_LONG_PULSE=90; // 180 degree turns

ServerSocket serv;
Socket sock;		// accepted connection
InputStream in;		// input of the accepted connection
OutputStream out;	// output of the accepted connection
ObjectOutputStream objOut;

byte buff[];		// used for reading/writing jpg file
int fileNum=1;		// used when we are searching for the files to send
Thread thr;		// thread that does image polling from the socket
String nextFile;	// this is the next file to be sent to the applet

public native int set(int servo, int position);
public native int ini(int comPort, int pulse);
public native void turnoff();
//public native void turnon();


	public static void main(String args[]){
		Jft639 jft = new Jft639();
		System.loadLibrary("ft639drv");
		System.out.println("Loaded library...");
		jft.go();
	}


	void go(){
		int val=0;
		try{
			serv = new ServerSocket(8000);
		} catch(Exception e){
			System.out.println("Error creating Server Socket");
			return;
		}
		ini(1, 90); // init servo motor
		startServer(); // wait for client ( applet ) to connect
		while(true){
			try{
				val = in.read();
				if( val == EOF ){
					System.out.println("EOF reached");
					throw new EOFException("EOF while reading slider value from input stream");
				}
			}catch(Exception e){
				System.out.println("Error reading from stream: "+e);
				startServer();
			}
			System.out.print(val + " ");
			set(0, (byte)val);
		}
	}


	boolean startServer(){
		System.out.println("Starting server...");
		try{
//			if(in){ in.close; in = null; }
//			if(out){ out.close; out = null; }
			if(sock!=null){ sock.close(); sock=null;}
			set(0,126);
//			turnoff(); // servo power off
			sock = serv.accept();
			System.out.println("Client connected...");
//			turnon(); // servo on
			in = sock.getInputStream();
			out = sock.getOutputStream();
			objOut = new ObjectOutputStream(out);
			thr = new Thread(this);
			thr.start(); // and start sending images
		} catch(Exception e){
			System.out.println("Error accepting connection: "+e);
			return false;
		}
		return true;
	}

	public void run(){ // will run till client (applet) disconnects and continuously send images
	String fileName="";
		String imgDir = "c:\\AutoCapture\\";
		findFirstFile(imgDir); // this will set fileNum
		while(sock!=null){
			FileInputStream fin = findNextFile(imgDir);
			if(fin == null) { break; }
			writeJpeg(fin);
			System.out.println("Deleting: "+nextFile);
			(new File(nextFile)).delete();
		}
	}


	void writeJpeg(FileInputStream fileIn){
		System.out.print(".");
		try{
			int available = fileIn.available();
			System.out.println("Sending file size: "+available);
			if (buff == null || buff.length < available){
				System.out.println("Resizing buffer to "+available);
				buff = new byte[available];
			}
			int rdBytes = fileIn.read(buff);
			fileIn.close();
			if(rdBytes!= available){
				System.out.println("Warning: could not read all bytes available in a file");
			}
			objOut.writeInt(rdBytes);	// write buffer lenght
			System.out.println("Wrote size "+rdBytes);
			objOut.write(buff,0,rdBytes);	// and then buffer itself
			System.out.println("Wrote "+rdBytes+ " bytes to socket");
//if(rdBytes>200000){ sleep(1000*30); }
//		} catch(IIOException imge){
		}catch(Exception e){
			System.out.println("While reading / writing jpeg: "+e);
		}
	}



	int getFileNum(String f){
		String noExt = f.substring(0, f.lastIndexOf('.') );
		try{
			return Integer.parseInt(noExt); // we need one after the last one
		} catch(Exception e){
			System.out.println("Could not scan file number");
			return 0;
		}
	}

	void findFirstFile(String dir){
		File [] files = (new File(dir)).listFiles();
		System.out.println("While searching for files found "+files.length+" entries");
		if(files.length == 0){
			fileNum=1;
			return;
		}
		for(int i =0; i < files.length; i++){ // delete all existing files
			System.out.println("Deleting " + files[i].getName() );
			files[i].delete(); // cleanup the directory 
		}
		System.out.println("Waiting for files to be captured");
		do{
			files = (new File(dir)).listFiles();
		}while(files.length == 0);
		int fileNum = getFileNum( files[0].getName() );
	}


	FileInputStream findNextFile(String dir){
		nextFile = dir+fileNum+".jpg";
		fileNum++;
		while(true){  // we will not return while exceptions are thrown
			try{
				if(sock == null){ return null; }
				System.out.println("Opening "+nextFile);
				return new FileInputStream(nextFile);
			} catch (FileNotFoundException e){
//				System.out.print(".");
			}
			try{ thr.sleep(900); } catch(Exception ex){
				System.out.println("Exception while sleeping: "+ex);
			}
		}
	}

}